@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: David Zeuthen <davidz@redhat.com>
  */
@@ -224,6 +224,10 @@ update_volume (GVfsUDisks2Volume *volume)
       GVariantIter iter;
       const gchar *configuration_type;
       GVariant *configuration_value;
+      UDisksLoop *loop = NULL;
+
+      loop = udisks_client_get_loop_for_block (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
+                                               volume->block);
 
       /* If unlocked, use the values from the unlocked block device for presentation */
       cleartext_block = udisks_client_get_cleartext_block (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
@@ -355,6 +359,17 @@ update_volume (GVfsUDisks2Volume *volume)
           g_object_unref (udisks_drive);
         }
 
+      /* Also automount loop devices set up by the user himself - e.g. via the
+       * udisks interfaces or the gnome-disk-image-mounter(1) command
+       */
+      if (loop != NULL)
+        {
+          if (udisks_loop_get_setup_by_uid (loop) == getuid ())
+            {
+              volume->should_automount = TRUE;
+            }
+        }
+
       /* Use hints, if available */
       hint = udisks_block_get_hint_name (volume->block);
       if (hint != NULL && strlen (hint) > 0)
@@ -383,11 +398,15 @@ update_volume (GVfsUDisks2Volume *volume)
         }
 
       /* Add an emblem, depending on whether the encrypted volume is locked or unlocked */
-      if (g_strcmp0 (udisks_block_get_id_type (volume->block), "crypto_LUKS") == 0 && volume->icon != NULL)
+      if (g_strcmp0 (udisks_block_get_id_type (volume->block), "crypto_LUKS") == 0)
         {
           GEmblem *emblem;
           GIcon *padlock;
           GIcon *emblemed_icon;
+
+          if (volume->icon == NULL)
+            volume->icon = g_themed_icon_new ("drive-removable-media");
+
           if (cleartext_block != NULL)
             padlock = g_themed_icon_new ("changes-allow");
           else
@@ -402,6 +421,7 @@ update_volume (GVfsUDisks2Volume *volume)
         }
 
       g_clear_object (&cleartext_block);
+      g_clear_object (&loop);
     }
   else
     {
@@ -885,6 +905,28 @@ mount_command_cb (GObject       *source_object,
 
 /* ------------------------------ */
 
+static void
+ensure_autoclear (MountData *data)
+{
+  UDisksLoop *loop;
+  loop = udisks_client_get_loop_for_block (gvfs_udisks2_volume_monitor_get_udisks_client (data->volume->monitor),
+                                           data->volume->block);
+  if (loop != NULL)
+    {
+      if (!udisks_loop_get_autoclear (loop) && udisks_loop_get_setup_by_uid (loop) == getuid ())
+        {
+          /* we don't care about the result */
+          udisks_loop_call_set_autoclear (loop, TRUE,
+                                          g_variant_new ("a{sv}", NULL), /* options */
+                                          NULL, NULL, NULL);
+        }
+      g_object_unref (loop);
+    }
+}
+
+/* ------------------------------ */
+
+
 static void
 mount_cb (GObject       *source_object,
           GAsyncResult  *res,
@@ -906,6 +948,9 @@ mount_cb (GObject       *source_object,
     }
   else
     {
+      /* if mounting worked, ensure that the loop device goes away when unmounted */
+      ensure_autoclear (data);
+
       gvfs_udisks2_volume_monitor_update (data->volume->monitor);
       g_simple_async_result_complete (data->simple);
       g_free (mount_path);
@@ -1035,6 +1080,9 @@ unlock_cb (GObject       *source_object,
     {
       UDisksObject *object;
 
+      /* if unlocking worked, ensure that the loop device goes away when locked */
+      ensure_autoclear (data);
+
       gvfs_udisks2_volume_monitor_update (data->volume->monitor);
 
       object = udisks_client_peek_object (gvfs_udisks2_volume_monitor_get_udisks_client (data->volume->monitor),
@@ -1045,7 +1093,7 @@ unlock_cb (GObject       *source_object,
           g_simple_async_result_set_error (data->simple,
                                            G_IO_ERROR,
                                            G_IO_ERROR_FAILED,
-                                           _("The unlocked device does not have a recognizable filesystem on it"));
+                                           _("The unlocked device does not have a recognizable file system on it"));
           g_simple_async_result_complete (data->simple);
           mount_data_free (data);
           goto out;
